import math
import unittest

from gravy import score


class TestScoreFunction(unittest.TestCase):

    def test_valid_sequence(self):
        """Test a valid sequence with known values."""
        seq = "ACDEFGHIKLMNPQRSTVWY"
        expected_score = (
            1.8
            + 2.5
            - 3.5
            - 3.5
            + 2.8
            - 0.4
            - 3.2
            + 4.5
            - 3.9
            + 3.8
            + 1.9
            - 3.5
            - 1.6
            - 3.5
            - 4.5
            - 0.8
            - 0.7
            + 4.2
            - 0.9
            - 1.3
        ) / 20
        self.assertAlmostEqual(score(seq), expected_score)

    def test_sequence_with_gap(self):
        """Test a sequence with gaps ('-')."""
        seq = "ACDEFGHI-KLMNPQRSTVWY"
        expected_score = (
            1.8
            + 2.5
            - 3.5
            - 3.5
            + 2.8
            - 0.4
            - 3.2
            + 4.5
            - 3.9
            + 3.8
            + 1.9
            - 3.5
            - 1.6
            - 3.5
            - 4.5
            - 0.8
            - 0.7
            + 4.2
            - 0.9
            - 1.3
        ) / 20
        self.assertAlmostEqual(score(seq), expected_score)

    def test_sequence_with_unknown_values(self):
        """Test a sequence with unknown or unsupported values ('X')."""
        seq = "ACDEXFGHIKLMNPQRSTVWY"
        expected_score = (
            1.8
            + 2.5
            - 3.5
            - 3.5
            + 2.8
            - 0.4
            - 3.2
            + 4.5
            - 3.9
            + 3.8
            + 1.9
            - 3.5
            - 1.6
            - 3.5
            - 4.5
            - 0.8
            - 0.7
            + 4.2
            - 0.9
            - 1.3
        ) / 20
        self.assertAlmostEqual(score(seq), expected_score)

    def test_sequence_with_only_gaps(self):
        """Test a sequence with only gaps ('-'), which should return NaN."""
        seq = "---"
        result = score(seq)
        self.assertTrue(math.isnan(result))

    def test_sequence_with_only_unknown_values(self):
        """Test a sequence with only unknown values ('X'), which should return NaN."""
        seq = "XXX"
        result = score(seq)
        self.assertTrue(math.isnan(result))

    def test_empty_sequence(self):
        """Test an empty sequence, which should return NaN."""
        seq = ""
        result = score(seq)
        self.assertTrue(math.isnan(result))

    def test_single_known_value(self):
        """Test a sequence with a single known value."""
        seq = "A"
        self.assertAlmostEqual(score(seq), 1.8)

    def test_single_unknown_value(self):
        """Test a sequence with a single unknown value ('X'), which should return NaN."""
        seq = "X"
        result = score(seq)
        self.assertTrue(math.isnan(result))


if __name__ == "__main__":
    unittest.main()



import enum
import functools
import math
import tomllib
from importlib import resources
from typing import *

import click
import preparse

__all__ = ["calculate", "main", "score"]


class Util(enum.Enum):
    util = None

    @functools.cached_property
    def data(self: Self) -> dict:
        "This cached property holds the cfg data."
        text: str = resources.read_text("gravy.core", "cfg.toml")
        ans: dict = tomllib.loads(text)
        return ans


def score(seq: Iterable) -> float:
    "This function calculates the GRAVY score."
    l: list = list()
    x: Any
    y: Any
    for x in seq:
        y = Util.util.data["values"][str(x)]
        if not math.isnan(y):
            l.append(y)
    if len(l):
        return sum(l) / len(l)
    else:
        return float("nan")


calculate = score  # for legacy


@preparse.PreParser().click()
@click.command(add_help_option=False)
@click.option(
    "--format",
    "f",
    help="format of the output",
    default=".5f",
    show_default=True,
)
@click.help_option("-h", "--help")
@click.version_option(None, "-V", "--version")
@click.argument("seq")
def main(seq: Iterable, f: str) -> None:
    "This command calculates the GRAVY score of seq."
    ans: float = score(seq)
    out: str = format(ans, f)
    click.echo(out)
